@@ -14,20 +14,24 @@
 
 #include <stdio.h>
 
-YClientContainer::YClientContainer(YWindow *parent, YFrameWindow *frame)
-:YWindow(parent)
+YClientContainer::YClientContainer(YWindow *parent, YFrameWindow *frame,
+                                   int depth, Visual *visual, Colormap cmap)
+    : YWindow(parent, None, depth, visual, cmap)
 {
     fFrame = frame;
     fHaveGrab = false;
     fHaveActionGrab = false;
 
-    setStyle(wsManager);
+    setStyle(wsManager | wsNoExpose);
     setDoubleBuffer(false);
     setPointer(YXApplication::leftPointer);
+    setTitle("Container");
+    show();
 }
 
 YClientContainer::~YClientContainer() {
-    releaseButtons();
+    if (destroyed() == false)
+        releaseButtons();
 }
 
 void YClientContainer::handleButton(const XButtonEvent &button) {
@@ -52,7 +56,7 @@ void YClientContainer::handleButton(const XButtonEvent &button) {
                 firstClick = true;
         }
     }
-#if 1
+
     if (clientMouseActions) {
         unsigned int k = button.button + XK_Pointer_Button1 - 1;
         unsigned int m = KEY_MODMASK(button.state);
@@ -81,7 +85,7 @@ void YClientContainer::handleButton(const XButtonEvent &button) {
             if ((doMove && getFrame()->canMove()) ||
                 (!doMove && getFrame()->canSize()))
             {
-                getFrame()->startMoveSize(doMove, 1,
+                getFrame()->startMoveSize(doMove, true,
                                           gx, gy,
                                           mx, my);
             }
@@ -92,25 +96,29 @@ void YClientContainer::handleButton(const XButtonEvent &button) {
             if (getFrame()->canMove()) {
                 int px = button.x + x();
                 int py = button.y + y();
-                getFrame()->startMoveSize(1, 1,
+                getFrame()->startMoveSize(true, true,
                                           0, 0,
                                           px, py);
             }
             return ;
-        } else if (IS_WMKEY(k, vm, gMouseWinRaise)) {
+        }
+        else if (gMouseWinRaise.eq(k, vm)
+            && (gMouseWinRaise != gMouseWinLower || getFrame()->canRaise()))
+        {
             XAllowEvents(xapp->display(), AsyncPointer, CurrentTime);
             getFrame()->wmRaise();
             return ;
         }
+        else if (gMouseWinLower.eq(k, vm)) {
+            XAllowEvents(xapp->display(), AsyncPointer, CurrentTime);
+            getFrame()->wmLower();
+            return ;
+        }
     }
-#endif
-    ///!!! do this first?
-    if (doActivate) {
-        bool input = getFrame() ? getFrame()->getInputFocusHint() : true;
 
-        if (input)
-            getFrame()->activate();
-    }
+    ///!!! do this first?
+    if (doActivate && getFrame() != manager->getFocus())
+        getFrame()->activate();
     if (doRaise)
         getFrame()->wmRaise();
     ///!!! it might be nice if this was per-window option (app-request)
@@ -118,8 +126,7 @@ void YClientContainer::handleButton(const XButtonEvent &button) {
         XAllowEvents(xapp->display(), ReplayPointer, CurrentTime);
     else
         XAllowEvents(xapp->display(), AsyncPointer, CurrentTime);
-    XSync(xapp->display(), 0);
-    return ;
+    XSync(xapp->display(), False);
 }
 
 // manage button grab on frame window to capture clicks to client window
@@ -140,7 +147,17 @@ void YClientContainer::grabButtons() {
         fHaveGrab = true;
 
         XGrabButton(xapp->display(),
-                    AnyButton, AnyModifier,
+                    Button1, AnyModifier,
+                    handle(), True,
+                    ButtonPressMask,
+                    GrabModeSync, GrabModeAsync, None, None);
+        XGrabButton(xapp->display(),
+                    Button2, AnyModifier,
+                    handle(), True,
+                    ButtonPressMask,
+                    GrabModeSync, GrabModeAsync, None, None);
+        XGrabButton(xapp->display(),
+                    Button3, AnyModifier,
                     handle(), True,
                     ButtonPressMask,
                     GrabModeSync, GrabModeAsync, None, None);
@@ -151,14 +168,18 @@ void YClientContainer::releaseButtons() {
     if (fHaveGrab) {
         fHaveGrab = false;
 
-        XUngrabButton(xapp->display(), AnyButton, AnyModifier, handle());
+        XUngrabButton(xapp->display(), Button1, AnyModifier, handle());
+        XUngrabButton(xapp->display(), Button2, AnyModifier, handle());
+        XUngrabButton(xapp->display(), Button3, AnyModifier, handle());
         fHaveActionGrab = false;
     }
     grabActions();
 }
 
 void YClientContainer::regrabMouse() {
-    XUngrabButton(xapp->display(), AnyButton, AnyModifier, handle());
+    XUngrabButton(xapp->display(), Button1, AnyModifier, handle());
+    XUngrabButton(xapp->display(), Button2, AnyModifier, handle());
+    XUngrabButton(xapp->display(), Button3, AnyModifier, handle());
 
     if (fHaveActionGrab)  {
         fHaveActionGrab = false;
@@ -172,16 +193,19 @@ void YClientContainer::regrabMouse() {
 }
 
 void YClientContainer::grabActions() {
-    if (clientMouseActions) {
-        if (!fHaveActionGrab) {
-            fHaveActionGrab = true;
-            if (gMouseWinMove.key != 0)
-                grabVButton(gMouseWinMove.key - XK_Pointer_Button1 + 1, gMouseWinMove.mod);
-            if (gMouseWinSize.key != 0)
-                grabVButton(gMouseWinSize.key - XK_Pointer_Button1 + 1, gMouseWinSize.mod);
-            if (gMouseWinRaise.key != 0)
-                grabVButton(gMouseWinRaise.key - XK_Pointer_Button1 + 1, gMouseWinRaise.mod);
-        }
+    if (clientMouseActions && fHaveActionGrab == false) {
+        fHaveActionGrab = true;
+        const KeySym minButton = XK_Pointer_Button1;
+        const KeySym maxButton = XK_Pointer_Button3;
+        const KeySym xkButton0 = XK_Pointer_Button1 - 1;
+        if (inrange(gMouseWinMove.key, minButton, maxButton))
+            grabVButton(gMouseWinMove.key - xkButton0, gMouseWinMove.mod);
+        if (inrange(gMouseWinSize.key, minButton, maxButton))
+            grabVButton(gMouseWinSize.key - xkButton0, gMouseWinSize.mod);
+        if (inrange(gMouseWinRaise.key, minButton, maxButton))
+            grabVButton(gMouseWinRaise.key - xkButton0, gMouseWinRaise.mod);
+        if (inrange(gMouseWinLower.key, minButton, maxButton))
+            grabVButton(gMouseWinLower.key - xkButton0, gMouseWinLower.mod);
     }
 }
 
@@ -191,18 +215,18 @@ void YClientContainer::handleConfigureRequest(const XConfigureRequestEvent &conf
     if (getFrame() &&
         configureRequest.window == getFrame()->client()->handle())
     {
-        XConfigureRequestEvent cre = configureRequest;
-
-        getFrame()->configureClient(cre);
+        getFrame()->configureClient(configureRequest);
     }
 }
 
 void YClientContainer::handleMapRequest(const XMapRequestEvent &mapRequest) {
     if (mapRequest.window == getFrame()->client()->handle()) {
+        manager->lockFocus();
         getFrame()->setState(WinStateMinimized |
                              WinStateHidden |
                              WinStateRollup,
                              0);
+        manager->unlockFocus();
         bool doActivate = true;
         getFrame()->updateFocusOnMap(doActivate);
         if (doActivate) {
@@ -220,8 +244,10 @@ void YClientContainer::handleCrossing(const XCrossingEvent &crossing) {
                  crossing.mode == NotifyNormal &&
                  manager->colormapWindow() == getFrame())
         {
-            manager->setColormapWindow(0);
+            manager->setColormapWindow(nullptr);
         }
     }
 }
 
+
+// vim: set sw=4 ts=4 et:
