@@ -5,13 +5,12 @@
  */
 #include "config.h"
 #include "ykey.h"
-#include "ypixbuf.h"
 #include "ybutton.h"
 #include "yaction.h"
 #include "ymenu.h"
 #include "yrect.h"
 #include "yicon.h"
-
+#include "wpixmaps.h"
 #include "yxapp.h" // !!! remove (AltMask)
 #include "yprefs.h"
 #include "prefs.h"
@@ -22,21 +21,16 @@
 
 #include "intl.h"
 
-YColor *YButton::normalButtonBg = 0;
-YColor *YButton::normalButtonFg = 0;
+YColorName YButton::normalButtonBg(&clrNormalButton);
+YColorName YButton::normalButtonFg(&clrNormalButtonText);
 
-YColor *YButton::activeButtonBg = 0;
-YColor *YButton::activeButtonFg = 0;
+YColorName YButton::activeButtonBg(&clrActiveButton);
+YColorName YButton::activeButtonFg(&clrActiveButtonText);
 
 ref<YFont> YButton::normalButtonFont;
 ref<YFont> YButton::activeButtonFont;
 
-// !!! needs to go away
-ref<YPixmap> taskbuttonPixmap;
-ref<YPixmap> taskbuttonactivePixmap;
-ref<YPixmap> taskbuttonminimizedPixmap;
-
-YButton::YButton(YWindow *parent, YAction *action, YMenu *popup) :
+YButton::YButton(YWindow *parent, YAction action, YMenu *popup) :
     YWindow(parent),
     fOver(false),
     fAction(action), fPopup(popup),
@@ -47,7 +41,7 @@ YButton::YButton(YWindow *parent, YAction *action, YMenu *popup) :
     fPressed(false),
     fEnabled(true),
     fHotCharPos(-1), hotKey(-1),
-    fListener(NULL),
+    fListener(nullptr),
     fSelected(false), fArmed(false),
     wasPopupActive(false),
     fPopupActive(false)
@@ -56,14 +50,6 @@ YButton::YButton(YWindow *parent, YAction *action, YMenu *popup) :
         normalButtonFont = YFont::getFont(XFA(normalButtonFontName));
     if (activeButtonFont == null)
         activeButtonFont = YFont::getFont(XFA(activeButtonFontName));
-    if (normalButtonBg == 0)
-        normalButtonBg = new YColor(clrNormalButton);
-    if (normalButtonFg == 0)
-        normalButtonFg = new YColor(clrNormalButtonText);
-    if (activeButtonBg == 0)
-        activeButtonBg = new YColor(clrActiveButton);
-    if (activeButtonFg == 0)
-        activeButtonFg = new YColor(clrActiveButtonText);
 }
 
 YButton::~YButton() {
@@ -73,6 +59,9 @@ YButton::~YButton() {
             removeAccelerator(hotKey, xapp->AltMask, this);
     }
     popdown();
+    if (fPopup && fPopup->isShared() == false) {
+        delete fPopup;
+    }
 }
 
 void YButton::paint(Graphics &g, int const d, const YRect &r) {
@@ -80,23 +69,22 @@ void YButton::paint(Graphics &g, int const d, const YRect &r) {
     YSurface surface(getSurface());
     g.drawSurface(surface, x, y, w, h);
 
-#ifndef LITE
-    if (fIcon != null)
+    if (fIcon != null) {
         fIcon->draw(g,
                     x + (w - fIconSize) / 2,
                     y + (h - fIconSize) / 2,
                     fIconSize);
-    else
-#endif
-    if (fImage != null)
+    }
+    else if (fImage != null) {
         g.drawImage(fImage,
                     x + (w - fImage->width()) / 2,
                     y + (h - fImage->height()) / 2);
+    }
     else if (fText != null) {
-        ref<YFont> font = fPressed ? activeButtonFont : normalButtonFont;
+        ref<YFont> font(getFont());
 
         int const w(font->textWidth(fText));
-        int const p((width() - w) / 2);
+        int const p((width() + 1 - w) / 2);
         int yp((height() - font->height()) / 2
                + font->ascent() + d);
 
@@ -105,18 +93,18 @@ void YButton::paint(Graphics &g, int const d, const YRect &r) {
         if (!fEnabled) {
             g.setColor(YColor::white);
             g.drawChars(fText, d + p + 1, yp + 1);
-            g.setColor(YColor::white->darker()->darker());
+            g.setColor(YColor::white->darker().darker());
             g.drawChars(fText, d + p, yp);
         } else {
             g.drawChars(fText, d + p, yp);
         }
-        if (fHotCharPos != -1)
+        if (fHotCharPos >= 0)
             g.drawCharUnderline(d + p, yp, fText, fHotCharPos);
     }
 }
 
 void YButton::paint(Graphics &g, const YRect &/*r*/) {
-    int d((fPressed || fArmed) ? 1 : 0);
+    int d(fPressed || fArmed);
     int x(0), y(0), w(width()), h(height());
 
     if (w > 1 && h > 1) {
@@ -129,8 +117,8 @@ void YButton::paint(Graphics &g, const YRect &/*r*/) {
         } else if (wmLook == lookGtk) {
             g.drawBorderG(x, y, w - 1, h - 1, !d);
             x += 1 + d; y += 1 + d; w -= 3; h -= 3;
-	} else if (wmLook == lookFlat){
-	    d = 0;
+        } else if (wmLook == lookFlat){
+            d = 0;
         } else {
             g.drawBorderW(x, y, w - 1, h - 1, !d);
             x += 1 + d; y += 1 + d; w -= 3; h -= 3;
@@ -141,11 +129,14 @@ void YButton::paint(Graphics &g, const YRect &/*r*/) {
         if (wmLook != lookFlat) {
             paintFocus(g, YRect(x, y, w, h));
         }
+        if (surface.gradient != null) {
+            g.maxOpacity();
+        }
     }
 }
 
 void YButton::paintFocus(Graphics &g, const YRect &/*r*/) {
-    int const d = (fPressed || fArmed ? 1 : 0);
+    int const d = (fPressed || fArmed);
     int const dp(wmLook == lookMetal ? 2 : 2 + d);
     int const ds(4);
 
@@ -157,11 +148,15 @@ void YButton::paintFocus(Graphics &g, const YRect &/*r*/) {
         g.setFunction(GXcopy);
         g.setPenStyle(false);
     } else {
+        short x1(dp), x2(dp), x3(dp+width()-ds-1), x4(dp);
+        short y1(dp), y2(dp+1), y3(dp+1), y4(dp+height()-ds-1);
+        unsigned short w1(width()-ds), w2(1), w3(1), w4(width()-ds);
+        unsigned short h1(1), h2(height()-ds-2), h3(height()-ds-2), h4(1);
         XRectangle focus[] = {
-            { dp, dp, width() - ds, 1 },
-            { dp, dp + 1, 1, height() - ds - 2 },
-            { dp + width() - ds - 1, dp + 1, 1, height() - ds - 2 },
-            { dp, dp + height() - ds - 1, width() - ds, 1 }
+            { x1, y1, w1, h1 },
+            { x2, y2, w2, h2 },
+            { x3, y3, w3, h3 },
+            { x4, y4, w4, h4 }
         };
 
         g.setClipRectangles(focus, 4);
@@ -209,7 +204,7 @@ void YButton::setArmed(bool armed, bool mouseDown) {
 }
 
 bool YButton::handleKey(const XKeyEvent &key) {
-    KeySym k = XKeycodeToKeysym(xapp->display(), (KeyCode)key.keycode, 0);
+    KeySym k = keyCodeToKeySym(key.keycode);
     unsigned m = KEY_MODMASK(key.state);
     int uk = (k < 256) ? ASCII::toUpper((char)k) : k;
 
@@ -236,10 +231,11 @@ bool YButton::handleKey(const XKeyEvent &key) {
                     bool wasArmed = fArmed;
 
                     // !!! is this guaranteed to work? (skip autorepeated keys)
-                    XEvent xev;
+                    XEvent xev = {};
 
-                    XCheckTypedWindowEvent(xapp->display(), handle(), KeyPress, &xev);
-                    if (xev.type == KeyPress &&
+                    if (XCheckTypedWindowEvent(xapp->display(), handle(),
+                                               KeyPress, &xev) &&
+                        xev.type == KeyPress &&
                         xev.xkey.time == key.time &&
                         xev.xkey.keycode == key.keycode &&
                         xev.xkey.state == key.state)
@@ -272,7 +268,7 @@ void YButton::updatePopup() {
 
 void YButton::handleButton(const XButtonEvent &button) {
     if (fEnabled) {
-        if (button.type == ButtonPress && button.button == 1) {
+        if (button.type == ButtonPress && button.button == Button1) {
             requestFocus(false);
             wasPopupActive = fArmed;
             setSelected(true);
@@ -286,23 +282,26 @@ void YButton::handleButton(const XButtonEvent &button) {
 
                 if ((!inWindow || wasPopupActive) && fArmed) {
                     setArmed(false, false);
-                    setSelected(false);
-                }
-            } else {
-                bool wasArmed = fArmed;
-
-                setArmed(false, false);
-                setSelected(false);
-                if (wasArmed) {
-                    actionPerformed(fAction, button.state);
-                    return ;
                 }
             }
+            setSelected(false);
         }
     }
     YWindow::handleButton(button);
 }
 
+void YButton::handleClick(const XButtonEvent &button, int count) {
+    if (fEnabled && fPopup == nullptr) {
+        bool wasArmed = fArmed;
+
+        setArmed(false, false);
+        setSelected(false);
+        if (wasArmed && count == 1 && button.button == Button1) {
+            actionPerformed(fAction, button.state);
+        }
+    }
+}
+
 void YButton::handleCrossing(const XCrossingEvent &crossing) {
     if (fSelected && fEnabled) {
         if (crossing.type == EnterNotify) {
@@ -325,20 +324,36 @@ void YButton::handleCrossing(const XCrossingEvent &crossing) {
     YWindow::handleCrossing(crossing);
 }
 
+ref<YFont> YButton::getActiveFont() {
+    return activeButtonFont;
+}
+
+YDimension YButton::getTextSize() {
+    if (fText != null) {
+        ref<YFont> font(getActiveFont());
+        return YDimension(font->textWidth(fText), font->height());
+    } else {
+        return YDimension(1, 1);
+    }
+}
+
 void YButton::updateSize() {
     int w = 72;
     int h = 18;
     if (fIcon != null) {
         w = h = fIconSize;
-    } else if (fImage != null) {
+    }
+    else if (fImage != null) {
         w = fImage->width();
         h = fImage->height();
-    } else if (fText != null) {
-        w = activeButtonFont->textWidth(fText);
-        h = activeButtonFont->ascent();
     }
-    setSize(w + 3 + 2 - ((wmLook == lookMetal || wmLook == lookFlat) ? 1 : 0),
-            h + 3 + 2 - ((wmLook == lookMetal || wmLook == lookFlat) ? 1 : 0));
+    else if (fText != null) {
+        YDimension d(getTextSize());
+        w = d.w;
+        h = d.h;
+    }
+    setSize(w + 3 + 2 - LOOK(lookMetal | lookFlat),
+            h + 3 + 2 - LOOK(lookMetal | lookFlat));
 }
 
 void YButton::setIcon(ref<YIcon> icon, int iconSize) {
@@ -357,34 +372,33 @@ void YButton::setImage(ref<YImage> image) {
     updateSize();
 }
 
-void YButton::setText(const ustring &str, int hotChar) {
+void YButton::setText(const mstring &str, int hotChar) {
     if (hotKey != -1) {
         removeAccelerator(hotKey, 0, this);
         if (xapp->AltMask != 0)
             removeAccelerator(hotKey, xapp->AltMask, this);
+        hotKey = -1;
     }
     fText = str;
     if (fText != null) {
         fHotCharPos = hotChar;
 
         if (fHotCharPos == -2) {
-            int i = fText.indexOf('_');
-            if (i != -1) {
-                fHotCharPos = i;
-                fText = fText.remove(i, 1);
+            fHotCharPos = fText.indexOf('_');
+            if (fHotCharPos >= 0) {
+                fText = fText.remove(fHotCharPos, 1);
             }
         }
 
-        hotKey = (fHotCharPos != -1) ? fText.charAt(fHotCharPos) : -1;
-        hotKey = ASCII::toUpper(hotKey);
-
+        hotKey = (fHotCharPos >= 0) ? fText.charAt(fHotCharPos) : -1;
         if (hotKey != -1) {
+            hotKey = ASCII::toUpper(hotKey);
+
             installAccelerator(hotKey, 0, this);
             if (xapp->AltMask != 0)
                 installAccelerator(hotKey, xapp->AltMask, this);
         }
-    } else
-        hotKey = -1;
+    }
     updateSize();
 }
 
@@ -402,8 +416,8 @@ void YButton::donePopup(YPopupWindow *popup) {
         return ;
     }
     popdown();
-    fArmed = 0;
-    fSelected = 0;
+    fArmed = false;
+    fSelected = false;
     repaint();
 }
 
@@ -416,7 +430,7 @@ void YButton::popup(bool mouseDown) {
         fPopup->setActionListener(getActionListener());
 
         int xiscreen = desktop->getScreenForRect(x, y, width(), height());
-        if (fPopup->popup(this, this, 0,
+        if (fPopup->popup(this, this, nullptr,
                           x - 2,
                           y + height(),
                           0,
@@ -439,12 +453,12 @@ bool YButton::isFocusTraversable() {
     return true;
 }
 
-void YButton::setAction(YAction *action) {
+void YButton::setAction(YAction action) {
     fAction = action;
 }
 
-void YButton::actionPerformed(YAction *action, unsigned modifiers) {
-    if (fListener && action && fEnabled)
+void YButton::actionPerformed(YAction action, unsigned modifiers) {
+    if (fListener && action != actionNull && fEnabled)
         fListener->actionPerformed(action, modifiers);
 }
 
@@ -452,18 +466,13 @@ ref<YFont> YButton::getFont() {
     return (fPressed ? activeButtonFont : normalButtonFont);
 }
 
-YColor * YButton::getColor() {
+YColor YButton::getColor() {
     return (fPressed ? activeButtonFg : normalButtonFg);
 }
 
 YSurface YButton::getSurface() {
-#ifdef CONFIG_GRADIENTS
     return (fPressed ? YSurface(activeButtonBg, buttonAPixmap, buttonAPixbuf)
                      : YSurface(normalButtonBg, buttonIPixmap, buttonIPixbuf));
-#else
-    return (fPressed ? YSurface(activeButtonBg, buttonAPixmap)
-                     : YSurface(normalButtonBg, buttonIPixmap));
-#endif
 }
 
 void YButton::setEnabled(bool enabled) {
@@ -477,3 +486,5 @@ void YButton::setEnabled(bool enabled) {
         repaint();
     }
 }
+
+// vim: set sw=4 ts=4 et:
