@@ -3,7 +3,6 @@
 
 #include "ypopup.h"
 #include "ytimer.h"
-#include "yarray.h"
 
 class YAction;
 class YActionListener;
@@ -11,7 +10,7 @@ class YMenuItem;
 
 class YMenu: public YPopupWindow, public YTimerListener {
 public:
-    YMenu(YWindow *parent = 0);
+    YMenu(YWindow *parent = nullptr);
     virtual ~YMenu();
 
     virtual void sizePopup(int hspace);
@@ -26,24 +25,33 @@ public:
     virtual void handleMotion(const XMotionEvent &motion);
     virtual void handleMotionOutside(bool top, const XMotionEvent &motion);
     virtual bool handleAutoScroll(const XMotionEvent &mouse);
+    virtual void configure(const YRect2& r2);
+    virtual void handleExpose(const XExposeEvent &expose) {}
+    virtual void repaint();
 
     void trackMotion(const int x_root, const int y_root, const unsigned state, bool submenu);
 
+    YMenuItem *add(YMenuItem *item, const char *icons);
+    YMenuItem *addItem(const mstring &name, int hotCharPos, const mstring &param, YAction action, const char *icons);
+    YMenuItem *addItem(const mstring &name, int hotCharPos, YAction action, YMenu *submenu, const char *icons);
+    YMenuItem *addSubmenu(const mstring &name, int hotCharPos, YMenu *submenu, const char *icons);
+
     YMenuItem *add(YMenuItem *item);
-    YMenuItem *addSorted(YMenuItem *item, bool duplicates);
-    YMenuItem *addItem(const char *name, int hotCharPos, const char *param, YAction *action);
-    YMenuItem *addItem(const char *name, int hotCharPos, YAction *action, YMenu *submenu);
-    YMenuItem *addSubmenu(const char *name, int hotCharPos, YMenu *submenu);
+    YMenuItem *addSorted(YMenuItem *item, bool duplicates, bool ignoreCase = false);
+    YMenuItem *addItem(const mstring &name, int hotCharPos, const mstring &param, YAction action);
+    YMenuItem *addItem(const mstring &name, int hotCharPos, YAction action, YMenu *submenu);
+    YMenuItem *addSubmenu(const mstring &name, int hotCharPos, YMenu *submenu);
     YMenuItem *addSeparator();
-    YMenuItem *addLabel(const char *name);
+    YMenuItem *addLabel(const mstring &name);
     void removeAll();
-    YMenuItem *findAction(const YAction *action);
+    YMenuItem *findAction(YAction action);
     YMenuItem *findSubmenu(const YMenu *sub);
-    YMenuItem *findName(const char *name, const int first = 0);
+    YMenuItem *findName(const mstring &name, const int first = 0);
     int findFirstLetRef(char firstLet, const int first, const int ignCase = 1);
 
-    void enableCommand(YAction *action); // 0 == All
-    void disableCommand(YAction *action); // 0 == All
+    void enableCommand(YAction action); // 0 == All
+    void disableCommand(YAction action); // 0 == All
+    void checkCommand(YAction action, bool check); // 0 == All
 
     int itemCount() const { return fItems.getCount(); }
     YMenuItem *getItem(int n) const { return fItems[n]; }
@@ -56,6 +64,7 @@ public:
     YActionListener *getActionListener() const { return fActionListener; }
 
     virtual bool handleTimer(YTimer *timer);
+    virtual void raise();
 
 private:
     YObjectArray<YMenuItem> fItems;
@@ -66,35 +75,36 @@ private:
     YPopupWindow *fPopup;
     YPopupWindow *fPopupActive;
     bool fShared;
+    bool fRaised;
     YActionListener *fActionListener;
     int activatedX, activatedY;
     int submenuItem;
 
-#ifdef CONFIG_GRADIENTS
-    ref<YPixbuf> fGradient;
-#endif
+    GraphicsBuffer fGraphics;
+    ref<YPixmap> fGradient;
+    ref<YPixmap> fMenusel;
 
     static YMenu *fPointedMenu;
-    static YTimer *fMenuTimer;
+    static lazy<YTimer> fMenuTimer;
     int fTimerX, fTimerY;
     int fTimerSubmenuItem;
     static int fAutoScrollDeltaX, fAutoScrollDeltaY;
     static int fAutoScrollMouseX, fAutoScrollMouseY;
 
     void getOffsets(int &left, int &top, int &right, int &bottom);
-    void getArea(int &x, int &y, int &w, int &h);
+    void getArea(int &x, int &y, unsigned &w, unsigned &h);
 
     void drawBackground(Graphics &g, int x, int y, int w, int h);
-    void drawSeparator(Graphics &g, int x, int y, int w);
+    void drawSeparator(Graphics &g, int x, int y, unsigned w);
 
-    void drawSubmenuArrow(Graphics &g, YMenuItem *mitem, 
+    void drawSubmenuArrow(Graphics &g, YMenuItem *mitem,
                           int left, int top);
     void paintItem(Graphics &g, const int i, const int l, const int t, const int r,
                    const int minY, const int maxY, bool draw);
 
     void repaintItem(int item);
     void paintItems();
-    int findItemPos(int item, int &x, int &y, int &h);
+    int findItemPos(int item, int &x, int &y, unsigned &h);
     int findItem(int x, int y);
     int findActiveItem(int cur, int direction);
     int findHotItem(char k);
@@ -106,20 +116,24 @@ private:
     int onCascadeButton(int selectedItem, int x, int y, bool checkPopup);
 
     void autoScroll(int deltaX, int deltaY, int mx, int my, const XMotionEvent *motion);
-    void finishPopup(YMenuItem *item, YAction *action, unsigned int modifiers);
+    void finishPopup(YMenuItem *item, YAction action, unsigned int modifiers);
     void hideSubmenu();
 };
 
+class LazyMenu {
+public:
+    virtual YMenu* ymenu() = 0;
+    virtual ~LazyMenu() {}
+};
+
 extern ref<YPixmap> menubackPixmap;
 extern ref<YPixmap> menuselPixmap;
 extern ref<YPixmap> menusepPixmap;
 
-#ifdef CONFIG_GRADIENTS
-class YPixbuf;
-
-extern ref<YPixbuf> menubackPixbuf;
-extern ref<YPixbuf> menuselPixbuf;
-extern ref<YPixbuf> menusepPixbuf;
-#endif
+extern ref<YImage> menubackPixbuf;
+extern ref<YImage> menuselPixbuf;
+extern ref<YImage> menusepPixbuf;
 
 #endif
+
+// vim: set sw=4 ts=4 et:
